# Install and configure Comptoir Du Libre
#
# @summary Install and configure Comptoir Du Libre
#
# @param comptoir_user Username of Unix account holding Comptoir du Libre application
# @param comptoir_repos_url URL of Comptoir repos in HTTPS mode (not in git mode)
# @param comptoir_srv_dir Absolute path where Comptoir source code should be cloned, example: '/home/comptoir/Comptoir-srv'. Must be coherent with comptoir_repos_url
# @param comptoir_revision Revision of Comptoir repos as understood by module VCSRepos, may be a branch, a tag, a SHA-1
# @param comptoir_db_user Database username owning the database for the application
# @param comptoir_db_user Database username owning
# @param comptoir_db_password PAssword for the user of the database
# @param comptoir_db_host Name of the database host, must an FQDN or an IP address
# @param comptoir_db_port Port of the database host, must an integer
#
# @example
#   include comptoir
class comptoir (
  String $comptoir_user                = 'comptoir',
  # Stdlib::Absolutepath $comptoir_dir   = '/home/comptoir/Comptoir-srv',
  String $comptoir_dir                 = '/home/comptoir/Comptoir-srv',
  # TODO grab this info from the repos URL, use Stdlib::basename
  Stdlib::Httpsurl $comptoir_repos_url = 'https://gitlab.adullact.net/Comptoir/Comptoir-srv.git',
  String $comptoir_revision            = 'master',
  String $comptoir_db_user             = 'comptoir',
  String $comptoir_db_name             = 'comptoir',
  String $comptoir_db_password         = 'comptoir',
  String $comptoir_db_host             = 'localhost',
  String $comptoir_db_port             = '5432',
  Boolean $debug                       = false,
) {

  # Inner variables
  $_sql_create_tables_and_procedure = 'config/SQL/COMPTOIR_DB_create_tables_and_procedures.sql'
  $_file_app_php = "${comptoir_dir}/config/app.php"
  $_file_comptoir_php = "${comptoir_dir}/config/comptoir.php"

  # Grab source code
  vcsrepo { $comptoir_dir:
    ensure   => present,
    provider => git,
    source   => $comptoir_repos_url,
    revision => $comptoir_revision,
    user     => $comptoir_user,
    force    => true, # See https://forge.puppet.com/puppetlabs/vcsrepo#force
  }

  # Create SQL tables & procedures
  # /!\ This is NOT idempotent => improve it :)
  # TODO Improve puppet code to be idempotent:
  # TODO 1. by creating an idempotent SQL, see https://stackoverflow.com/questions/33631186/how-do-i-make-alter-column-idempotent
  # TODO 2. by adding a directive unless or onlyif

  postgresql_psql { 'Create SQL tables & procedures':
    command   => "\\i ${comptoir_dir}/${_sql_create_tables_and_procedure}",
    db        => $comptoir_db_name,
    psql_user => $comptoir_db_user,
  }

  # Install app with Composer
  -> Exec { 'Install app with Composer':
    command     => '/usr/local/bin/composer --no-progress install',
    cwd         => $comptoir_dir,
    user        => $comptoir_user,
    environment => [ 'HOME=/home/comptoir' ],
    timeout     => 1200, # default value (300s) * 4
  }

  # Comptoir configuration: APP.PHP
  -> if $debug {
    Exec { 'Comptoir enable debug':
      command     => "/bin/sed -i -e 's!//COMPTOIR-DEBUG!!' ${_file_app_php}",
      cwd         => $comptoir_dir,
      user        => $comptoir_user,
      environment => [ 'HOME=/home/comptoir' ],
      onlyif      => "/bin/grep '//COMPTOIR-DEBUG' ${_file_app_php}",
    }
  }

  # Comptoir configuration: COMPTOIR.PHP

}
